#!/usr/bin/python

import os
import sys

if sys.hexversion >= 0x3000000:
	long = int

if len(sys.argv) != 4:
	sys.stderr.write("need 3 args, first being directory to scan, " + \
		"second being latest, third being it's size\n")
	sys.exit(1)

path = sys.argv[1]
maxsize = long(sys.argv[3])
if len(sys.argv[2]) != 8:
	sys.stderr.write("date arg must be YYYYMMDD\n")
	sys.exit(1)
root = sys.argv[2]

file_list = [x for x in os.listdir(path) if os.path.isfile(os.path.join(path, x))]
file_list.sort()
pref_name = "snapshot-"
post_name = ".patch.bz2"
deltas = [x for x in file_list if x.startswith(pref_name) and x.endswith(post_name)]
chains = {}
for x in deltas:
	try:
		start, end = x[len(pref_name):-len(post_name)].split("-")
	except ValueError:
		continue
	if end in chains:
		sys.stderr.write("warning, end %s already exists; this is a bit weird\n" % end)
		continue
	chains[end] = start	

if root not in chains:
	sys.stderr.write(("error: previous delta for '%s' does not exist, " + \
		"so none will be removed\n") % (root,))
	sys.exit(1)

save_files = set()
node = root
count = 0
while node in chains:
	s = "%s%s-%s%s" % (pref_name, chains[node], node, post_name)
	count += os.stat(os.path.join(path, s)).st_size
	if count > maxsize:
		break
	save_files.update([s, s+".md5sum", s+".gpgsig"])
	node = chains[node]

for x in file_list:
	if x not in save_files:
		sys.stdout.write("nuking %s\n" % x)
		try:
			os.unlink(os.path.join(path,x))
		except OSError as oe:
			sys.stderr.write("failed removing %s cause of %s\n" % \
				(os.path.join(path, x), str(oe)))
